=== modified file 'HACKING'
--- HACKING	2006-11-03 01:52:12 +0000
+++ HACKING	2007-01-09 15:19:18 +0000
@@ -209,6 +209,9 @@
 
 Consider naming classes as nouns and functions/methods as verbs.
 
+Try to avoid using abbreviations in names, because there can be
+inconsistency if other people use the full name.
+
 
 Standard names
 --------------

=== modified file 'NEWS'
--- NEWS	2007-01-04 23:36:44 +0000
+++ NEWS	2007-01-18 04:11:51 +0000
@@ -1,4 +1,37 @@
 IN DEVELOPMENT
+  IMPROVEMENTS:
+
+    * ``bzr mv`` enhanced to support already moved files.
+      In the past the mv command would have failed if the source file doesn't
+      exist. In this situation ``bzr mv`` would now detect that the file has
+      already moved and update the repository accordingly, if the target file
+      does exist.
+      A new option ``--after`` has been added so that if two files already
+      exist, you could notify Bazaar that you have moved a (versioned) file and
+      replaced it with another. Thus in this case ``bzr move --after`` will
+      only update the Bazaar identifier.
+      (Steffen Eichenberg, Marius Kruger)
+
+    * ``ls`` now works on treeless branches and remote branches.
+      (Aaron Bentley)
+
+    * ``bzr help global-options`` describes the global options.
+      (Aaron Bentley)
+
+  INTERNALS:
+
+    * Reserved ids (any revision-id ending in a colon) are rejected by
+      versionedfiles, repositories, branches, and working trees
+      (Aaron Bentley)
+
+  BUGFIXES:
+
+  TESTING:
+
+    * New ``--first`` option to ``bzr selftest`` to run specified tests
+      before the rest of the suite.  (Martin Pool)
+
+bzr 0.14rc1  2007-01-16
 
   IMPROVEMENTS:
 

=== modified file 'bzr'
--- bzr	2007-01-24 17:34:01 +0000
+++ bzr	2007-01-24 20:40:20 +0000
@@ -84,7 +84,7 @@
 import bzrlib.commands
 import bzrlib.trace
 
-if bzrlib.version_info[:3] != (0, 14, 0):
+if bzrlib.version_info[:3] != (0, 15, 0):
     sys.stderr.write("bzr: WARNING: bzrlib version doesn't match the bzr program.\n"
             "This may indicate an installation problem.\n"
             "bzrlib from %s is version %r\n"

=== modified file 'bzrlib/__init__.py'
--- bzrlib/__init__.py	2006-12-01 19:41:16 +0000
+++ bzrlib/__init__.py	2007-01-16 15:56:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@
 user_encoding = get_user_encoding()
 
 
-__copyright__ = "Copyright 2005, 2006 Canonical Ltd."
+__copyright__ = "Copyright 2005, 2006, 2007 Canonical Ltd."
 
 # same format as sys.version_info: "A tuple containing the five components of
 # the version number: major, minor, micro, releaselevel, and serial. All
@@ -35,7 +35,7 @@
 # Python version 2.0 is (2, 0, 0, 'final', 0)."  Additionally we use a
 # releaselevel of 'dev' for unreleased under-development code.
 
-version_info = (0, 14, 0, 'dev', 0)
+version_info = (0, 15, 0, 'dev', 0)
 
 if version_info[3] == 'final':
     version_string = '%d.%d.%d' % version_info[:3]

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2006-12-07 16:21:39 +0000
+++ bzrlib/branch.py	2007-01-17 15:37:08 +0000
@@ -1077,6 +1077,7 @@
     def append_revision(self, *revision_ids):
         """See Branch.append_revision."""
         for revision_id in revision_ids:
+            _mod_revision.check_not_reserved_id(revision_id)
             mutter("add {%s} to revision-history" % revision_id)
         rev_history = self.revision_history()
         rev_history.extend(revision_ids)

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2007-01-05 17:12:03 +0000
+++ bzrlib/builtins.py	2007-01-24 17:06:02 +0000
@@ -51,7 +51,7 @@
 """)
 
 from bzrlib.commands import Command, display_command
-from bzrlib.option import Option
+from bzrlib.option import Option, RegistryOption
 from bzrlib.progress import DummyProgress, ProgressPhase
 from bzrlib.trace import mutter, note, log_error, warning, is_quiet, info
 
@@ -450,16 +450,25 @@
 
     If the last argument is a versioned directory, all the other names
     are moved into it.  Otherwise, there must be exactly two arguments
-    and the file is changed to a new name, which must not already exist.
+    and the file is changed to a new name.
+
+    If OLDNAME does not exist on the filesystem but is versioned and
+    NEWNAME does exist on the filesystem but is not versioned, mv
+    assumes that the file has been manually moved and only updates
+    its internal inventory to reflect that change.
+    The same is valid when moving many SOURCE files to a DESTINATION.
 
     Files cannot be moved between branches.
     """
 
     takes_args = ['names*']
+    takes_options = [Option("after", help="move only the bzr identifier"
+        " of the file (file has already been moved). Use this flag if"
+        " bzr is not able to detect this itself.")]
     aliases = ['move', 'rename']
     encoding_type = 'replace'
 
-    def run(self, names_list):
+    def run(self, names_list, after=False):
         if names_list is None:
             names_list = []
 
@@ -469,13 +478,14 @@
         
         if os.path.isdir(names_list[-1]):
             # move into existing directory
-            for pair in tree.move(rel_names[:-1], rel_names[-1]):
+            for pair in tree.move(rel_names[:-1], rel_names[-1], after=after):
                 self.outf.write("%s => %s\n" % pair)
         else:
             if len(names_list) != 2:
-                raise errors.BzrCommandError('to mv multiple files the destination '
-                                             'must be a versioned directory')
-            tree.rename_one(rel_names[0], rel_names[1])
+                raise errors.BzrCommandError('to mv multiple files the'
+                                             ' destination must be a versioned'
+                                             ' directory')
+            tree.rename_one(rel_names[0], rel_names[1], after=after)
             self.outf.write("%s => %s\n" % (rel_names[0], rel_names[1]))
             
     
@@ -1079,12 +1089,14 @@
     """
     takes_args = ['location?']
     takes_options = [
-                     Option('format', 
+                     RegistryOption('format',
                             help='Specify a format for this branch. Current'
                                  ' formats are: default, knit, metaweave and'
                                  ' weave. Default is knit; metaweave and'
                                  ' weave are deprecated',
-                            type=get_format_type),
+                            registry=bzrdir.format_registry,
+                            converter=get_format_type,
+                            value_switches=True),
                      ]
     def run(self, location=None, format=None):
         if format is None:
@@ -1137,12 +1149,14 @@
         (add files here)
     """
     takes_args = ["location"] 
-    takes_options = [Option('format', 
+    takes_options = [RegistryOption('format',
                             help='Specify a format for this repository.'
                                  ' Current formats are: default, knit,'
                                  ' metaweave and weave. Default is knit;'
                                  ' metaweave and weave are deprecated',
-                            type=get_format_type),
+                            registry=bzrdir.format_registry,
+                            converter=get_format_type,
+                            value_switches=True),
                      Option('trees',
                              help='Allows branches in repository to have'
                              ' a working tree')]
@@ -1545,14 +1559,17 @@
                                              ' and PATH')
             fs_path = path
             prefix = path
-        tree, relpath = WorkingTree.open_containing(fs_path)
+        tree, branch, relpath = bzrdir.BzrDir.open_containing_tree_or_branch(
+            fs_path)
         if from_root:
             relpath = u''
         elif relpath:
             relpath += '/'
         if revision is not None:
-            tree = tree.branch.repository.revision_tree(
-                revision[0].in_history(tree.branch).rev_id)
+            tree = branch.repository.revision_tree(
+                revision[0].in_history(branch).rev_id)
+        elif tree is None:
+            tree = branch.basis_tree()
 
         for fp, fc, fkind, fid, entry in tree.list_files(include_root=False):
             if fp.startswith(relpath):
@@ -1964,12 +1981,14 @@
     """
     takes_args = ['url?']
     takes_options = [
-                     Option('format', 
-                            help='Upgrade to a specific format. Current formats'
-                                 ' are: default, knit, metaweave and weave.'
-                                 ' Default is knit; metaweave and weave are'
-                                 ' deprecated',
-                            type=get_format_type),
+                    RegistryOption('format',
+                        help='Upgrade to a specific format. Current formats'
+                             ' are: default, knit, metaweave and weave.'
+                             ' Default is knit; metaweave and weave are'
+                             ' deprecated',
+                        registry=bzrdir.format_registry,
+                        converter=get_format_type,
+                        value_switches=True),
                     ]
 
 
@@ -2047,22 +2066,29 @@
 class cmd_selftest(Command):
     """Run internal test suite.
     
-    This creates temporary test directories in the working directory,
-    but not existing data is affected.  These directories are deleted
-    if the tests pass, or left behind to help in debugging if they
-    fail and --keep-output is specified.
+    This creates temporary test directories in the working directory, but not
+    existing data is affected.  These directories are deleted if the tests
+    pass, or left behind to help in debugging if they fail and --keep-output
+    is specified.
     
-    If arguments are given, they are regular expressions that say
-    which tests should run.
+    If arguments are given, they are regular expressions that say which tests
+    should run.  Tests matching any expression are run, and other tests are
+    not run.
+
+    Alternatively if --first is given, matching tests are run first and then
+    all other tests are run.  This is useful if you have been working in a
+    particular area, but want to make sure nothing else was broken.
 
     If the global option '--no-plugins' is given, plugins are not loaded
     before running the selftests.  This has two effects: features provided or
     modified by plugins will not be tested, and tests provided by plugins will
     not be run.
 
-    examples:
+    examples::
         bzr selftest ignore
+            run only tests relating to 'ignore'
         bzr --no-plugins selftest -v
+            disable plugins and list tests as they're run
     """
     # TODO: --list should give a list of all available tests
 
@@ -2103,12 +2129,16 @@
                      Option('clean-output',
                             help='clean temporary tests directories'
                                  ' without running tests'),
+                     Option('first',
+                            help='run all tests, but run specified tests first',
+                            )
                      ]
     encoding_type = 'replace'
 
     def run(self, testspecs_list=None, verbose=None, one=False,
             keep_output=False, transport=None, benchmark=None,
-            lsprof_timed=None, cache_dir=None, clean_output=False):
+            lsprof_timed=None, cache_dir=None, clean_output=False,
+            first=False):
         import bzrlib.ui
         from bzrlib.tests import selftest
         import bzrlib.benchmarks as benchmarks
@@ -2147,7 +2177,9 @@
                               transport=transport,
                               test_suite_factory=test_suite_factory,
                               lsprof_timed=lsprof_timed,
-                              bench_history=benchfile)
+                              bench_history=benchfile,
+                              matching_tests_first=first,
+                              )
         finally:
             if benchfile is not None:
                 benchfile.close()
@@ -2599,7 +2631,10 @@
             other_branch = parent
             if other_branch is None:
                 raise errors.BzrCommandError("No peer location known or specified.")
-            print "Using last location: " + local_branch.get_parent()
+            display_url = urlutils.unescape_for_display(parent,
+                                                        self.outf.encoding)
+            print "Using last location: " + display_url
+
         remote_branch = Branch.open(other_branch)
         if remote_branch.base == local_branch.base:
             remote_branch = local_branch

=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2007-01-09 14:08:26 +0000
+++ bzrlib/bzrdir.py	2007-01-17 14:33:15 +0000
@@ -558,6 +558,26 @@
                 raise errors.NotBranchError(path=url)
             a_transport = new_t
 
+    @classmethod
+    def open_containing_tree_or_branch(klass, location):
+        """Return the branch and working tree contained by a location.
+
+        Returns (tree, branch, relpath).
+        If there is no tree at containing the location, tree will be None.
+        If there is no branch containing the location, an exception will be
+        raised
+        relpath is the portion of the path that is contained by the branch.
+        """
+        bzrdir, relpath = klass.open_containing(location)
+        try:
+            tree = bzrdir.open_workingtree()
+        except (errors.NoWorkingTree, errors.NotLocalUrl):
+            tree = None
+            branch = bzrdir.open_branch()
+        else:
+            branch = tree.branch
+        return tree, branch, relpath
+
     def open_repository(self, _unsupported=False):
         """Open the repository object at this BzrDir if one is present.
 

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2007-01-04 20:59:46 +0000
+++ bzrlib/errors.py	2007-01-19 19:23:53 +0000
@@ -18,12 +18,17 @@
 """
 
 
-from bzrlib import symbol_versioning
-from bzrlib.patches import (PatchSyntax, 
-                            PatchConflict, 
-                            MalformedPatchHeader,
-                            MalformedHunkHeader,
-                            MalformedLine,)
+from bzrlib import (
+    osutils,
+    symbol_versioning,
+    )
+from bzrlib.patches import (
+    MalformedHunkHeader,
+    MalformedLine,
+    MalformedPatchHeader,
+    PatchConflict,
+    PatchSyntax,
+    )
 
 
 # TODO: is there any value in providing the .args field used by standard
@@ -199,6 +204,12 @@
         self.revision_id = revision_id
         self.branch = branch
 
+class ReservedId(BzrError):
+
+    _fmt = "Reserved revision-id {%(revision_id)s}"
+
+    def __init__(self, revision_id):
+        self.revision_id = revision_id
 
 class NoSuchId(BzrError):
 
@@ -288,6 +299,14 @@
 
     _fmt = "Error in command line options"
 
+
+class BadOptionValue(BzrError):
+
+    _fmt = """Bad value "%(value)s" for option "%(name)s"."""
+
+    def __init__(self, name, value):
+        BzrError.__init__(self, name=name, value=value)
+
     
 class StrictCommitFailed(BzrError):
 
@@ -319,6 +338,32 @@
     _fmt = "File exists: %(path)r%(extra)s"
 
 
+class RenameFailedFilesExist(BzrError):
+    """Used when renaming and both source and dest exist."""
+
+    _fmt = ("Could not rename %(source)s => %(dest)s because both files exist."
+         "%(extra)s")
+
+    def __init__(self, source, dest, extra=None):
+        BzrError.__init__(self)
+        self.source = str(source)
+        self.dest = str(dest)
+        if extra:
+            self.extra = ' ' + str(extra)
+        else:
+            self.extra = ''
+
+
+class NotADirectory(PathError):
+
+    _fmt = "%(path)r is not a directory %(extra)s"
+
+
+class NotInWorkingDirectory(PathError):
+
+    _fmt = "%(path)r is not in the working directory %(extra)s"
+
+
 class DirectoryNotEmpty(PathError):
 
     _fmt = "Directory not empty: %(path)r%(extra)s"
@@ -491,17 +536,50 @@
         self.repo_format = repo_format
 
 
+class AlreadyVersionedError(BzrError):
+    """Used when a path is expected not to be versioned, but it is."""
+
+    _fmt = "%(context_info)s%(path)s is already versioned"
+
+    def __init__(self, path, context_info=None):
+        """Construct a new NotVersionedError.
+
+        :param path: This is the path which is versioned,
+        which should be in a user friendly form.
+        :param context_info: If given, this is information about the context,
+        which could explain why this is expected to not be versioned.
+        """
+        BzrError.__init__(self)
+        self.path = path
+        if context_info is None:
+            self.context_info = ''
+        else:
+            self.context_info = context_info + ". "
+
+
 class NotVersionedError(BzrError):
-
-    _fmt = "%(path)s is not versioned"
-
-    def __init__(self, path):
+    """Used when a path is expected to be versioned, but it is not."""
+
+    _fmt = "%(context_info)s%(path)s is not versioned"
+
+    def __init__(self, path, context_info=None):
+        """Construct a new NotVersionedError.
+
+        :param path: This is the path which is not versioned,
+        which should be in a user friendly form.
+        :param context_info: If given, this is information about the context,
+        which could explain why this is expected to be versioned.
+        """
         BzrError.__init__(self)
         self.path = path
+        if context_info is None:
+            self.context_info = ''
+        else:
+            self.context_info = context_info + ". "
 
 
 class PathsNotVersionedError(BzrError):
-    # used when reporting several paths are not versioned
+    """Used when reporting several paths which are not versioned"""
 
     _fmt = "Path(s) are not versioned: %(paths_as_string)s"
 
@@ -514,17 +592,21 @@
 
 class PathsDoNotExist(BzrError):
 
-    _fmt = "Path(s) do not exist: %(paths_as_string)s"
+    _fmt = "Path(s) do not exist: %(paths_as_string)s%(extra)s"
 
     # used when reporting that paths are neither versioned nor in the working
     # tree
 
-    def __init__(self, paths):
+    def __init__(self, paths, extra=None):
         # circular import
         from bzrlib.osutils import quotefn
         BzrError.__init__(self)
         self.paths = paths
         self.paths_as_string = ' '.join([quotefn(p) for p in paths])
+        if extra:
+            self.extra = ': ' + str(extra)
+        else:
+            self.extra = ''
 
 
 class BadFileKindError(BzrError):
@@ -1278,6 +1360,48 @@
     _fmt = "Moving the root directory is not supported at this time"
 
 
+class BzrMoveFailedError(BzrError):
+
+    _fmt = "Could not move %(from_path)s%(operator)s %(to_path)s%(extra)s"
+
+    def __init__(self, from_path='', to_path='', extra=None):
+        BzrError.__init__(self)
+        if extra:
+            self.extra = ': ' + str(extra)
+        else:
+            self.extra = ''
+
+        has_from = len(from_path) > 0
+        has_to = len(to_path) > 0
+        if has_from:
+            self.from_path = osutils.splitpath(from_path)[-1]
+        else:
+            self.from_path = ''
+
+        if has_to:
+            self.to_path = osutils.splitpath(to_path)[-1]
+        else:
+            self.to_path = ''
+
+        self.operator = ""
+        if has_from and has_to:
+            self.operator = " =>"
+        elif has_from:
+            self.from_path = "from " + from_path
+        elif has_to:
+            self.operator = "to"
+        else:
+            self.operator = "file"
+
+
+class BzrRenameFailedError(BzrMoveFailedError):
+
+    _fmt = "Could not rename %(from_path)s%(operator)s %(to_path)s%(extra)s"
+
+    def __init__(self, from_path, to_path, extra=None):
+        BzrMoveFailedError.__init__(self, from_path, to_path, extra)
+
+
 class BzrBadParameterNotString(BzrBadParameter):
 
     _fmt = "Parameter %(param)s is not a string or unicode string."

=== modified file 'bzrlib/help_topics.py'
--- bzrlib/help_topics.py	2007-01-10 06:01:49 +0000
+++ bzrlib/help_topics.py	2007-01-17 14:46:47 +0000
@@ -127,6 +127,31 @@
 """
 
 
+_global_options =\
+"""Global Options
+
+These options may be used with any command, and may appear in front of any
+command.  (e.g. "bzr --quiet help").
+
+--quiet        Suppress informational output; only print errors and warnings
+--version      Print the version number
+
+--no-aliases   Do not process command aliases when running this command
+--builtin      Use the built-in version of a command, not the plugin version.
+               This does not suppress other plugin effects
+--no-plugins   Do not process any plugins
+
+--Derror       Instead of normal error handling, always print a traceback on
+               error.
+--profile      Profile execution using the hotshot profiler
+--lsprof       Profile execution using the lsprof profiler
+--lsprof-file  Profile execution using the lsprof profiler, and write the
+               results to a specified file.
+
+Note: --version must be supplied before any command.
+"""
+
+
 topic_registry.register("revisionspec", _help_on_revisionspec,
                         "Explain how to use --revision")
 topic_registry.register('basic', _basic_help, "Basic commands")
@@ -135,3 +160,5 @@
     from bzrlib import bzrdir
     return bzrdir.format_registry.help_topic(topic)
 topic_registry.register('formats', get_format_topic, 'Directory formats')
+topic_registry.register('global-options', _global_options,
+                        'Options that can be used with any command')

=== modified file 'bzrlib/inventory.py'
--- bzrlib/inventory.py	2006-12-12 22:40:01 +0000
+++ bzrlib/inventory.py	2006-12-22 17:11:36 +0000
@@ -944,7 +944,7 @@
                 # if we finished all children, pop it off the stack
                 stack.pop()
 
-    def iter_entries_by_dir(self, from_dir=None):
+    def iter_entries_by_dir(self, from_dir=None, specific_file_ids=None):
         """Iterate over the entries in a directory first order.
 
         This returns all entries for a directory before returning
@@ -959,10 +959,34 @@
         if from_dir is None:
             if self.root is None:
                 return
+            # Optimize a common case
+            if specific_file_ids is not None and len(specific_file_ids) == 1:
+                file_id = list(specific_file_ids)[0]
+                if file_id in self:
+                    yield self.id2path(file_id), self[file_id]
+                return 
             from_dir = self.root
-            yield '', self.root
+            if (specific_file_ids is None or 
+                self.root.file_id in specific_file_ids):
+                yield '', self.root
         elif isinstance(from_dir, basestring):
             from_dir = self._byid[from_dir]
+
+        if specific_file_ids is not None:
+            parents = set()
+            def add_ancestors(file_id):
+                if file_id not in self:
+                    return
+                parent_id = self[file_id].parent_id
+                if parent_id is None:
+                    return
+                if parent_id not in parents:
+                    parents.add(parent_id)
+                    add_ancestors(parent_id)
+            for file_id in specific_file_ids:
+                add_ancestors(file_id)
+        else:
+            parents = None
             
         stack = [(u'', from_dir)]
         while stack:
@@ -973,10 +997,13 @@
 
                 child_relpath = cur_relpath + child_name
 
-                yield child_relpath, child_ie
+                if (specific_file_ids is None or 
+                    child_ie.file_id in specific_file_ids):
+                    yield child_relpath, child_ie
 
                 if child_ie.kind == 'directory':
-                    child_dirs.append((child_relpath+'/', child_ie))
+                    if parents is None or child_ie.file_id in parents:
+                        child_dirs.append((child_relpath+'/', child_ie))
             stack.extend(reversed(child_dirs))
 
     def entries(self):

=== modified file 'bzrlib/knit.py'
--- bzrlib/knit.py	2006-12-21 20:07:34 +0000
+++ bzrlib/knit.py	2007-01-17 15:37:08 +0000
@@ -518,6 +518,7 @@
 
     def get_delta(self, version_id):
         """Get a delta for constructing version from some other version."""
+        self.check_not_reserved_id(version_id)
         if not self.has_version(version_id):
             raise RevisionNotPresent(version_id, self.filename)
         
@@ -685,6 +686,7 @@
         ### FIXME escape. RBC 20060228
         if contains_whitespace(version_id):
             raise InvalidRevisionId(version_id, self.filename)
+        self.check_not_reserved_id(version_id)
         if self.has_version(version_id):
             raise RevisionAlreadyPresent(version_id, self.filename)
         self._check_lines_not_unicode(lines)
@@ -797,6 +799,8 @@
 
     def get_line_list(self, version_ids):
         """Return the texts of listed versions as a list of strings."""
+        for version_id in version_ids:
+            self.check_not_reserved_id(version_id)
         text_map, content_map = self._get_content_maps(version_ids)
         return [text_map[v] for v in version_ids]
 

=== modified file 'bzrlib/option.py'
--- bzrlib/option.py	2007-01-10 07:28:57 +0000
+++ bzrlib/option.py	2007-01-17 16:32:40 +0000
@@ -26,6 +26,7 @@
 from bzrlib import (
     errors,
     revisionspec,
+    symbol_versioning,
     )
 """)
 from bzrlib.trace import warning
@@ -109,6 +110,7 @@
             (typestring, type_list)
         raise errors.BzrCommandError(msg)
 
+
 class Option(object):
     """Description of a command line option
     
@@ -146,7 +148,18 @@
         self.argname = argname
 
     def short_name(self):
-        return self._short_name
+        if self._short_name:
+            return self._short_name
+        else:
+            # remove this when SHORT_OPTIONS is removed
+            # XXX: This is accessing a DeprecatedDict, so we call the super 
+            # method to avoid warnings
+            for (k, v) in dict.iteritems(Option.SHORT_OPTIONS):
+                if v == self:
+                    return k
+
+    def set_short_name(self, short_name):
+        self._short_name = short_name
 
     def get_negation_name(self):
         if self.name.startswith('no-'):
@@ -190,6 +203,75 @@
         yield self.name, self.short_name(), argname, self.help
 
 
+class RegistryOption(Option):
+    """Option based on a registry
+
+    The values for the options correspond to entries in the registry.  Input
+    must be a registry key.  After validation, it is converted into an object
+    using Registry.get or a caller-provided converter.
+    """
+
+    def validate_value(self, value):
+        """Validate a value name"""
+        if value not in self.registry:
+            raise errors.BadOptionValue(self.name, value)
+
+    def convert(self, value):
+        """Convert a value name into an output type"""
+        self.validate_value(value)
+        if self.converter is None:
+            return self.registry.get(value)
+        else:
+            return self.converter(value)
+
+    def __init__(self, name, help, registry, converter=None,
+        value_switches=False):
+        """
+        Constructor.
+
+        :param name: The option name.
+        :param help: Help for the option.
+        :param registry: A Registry containing the values
+        :param converter: Callable to invoke with the value name to produce
+            the value.  If not supplied, self.registry.get is used.
+        :param value_switches: If true, each possible value is assigned its
+            own switch.  For example, instead of '--format metaweave',
+            '--metaweave' can be used interchangeably.
+        """
+        Option.__init__(self, name, help, type=self.convert)
+        self.registry = registry
+        self.name = name
+        self.converter = converter
+        self.value_switches = value_switches
+
+    def add_option(self, parser, short_name):
+        """Add this option to an Optparse parser"""
+        Option.add_option(self, parser, short_name)
+        if self.value_switches:
+            for key in self.registry.keys():
+                option_strings = ['--%s' % key]
+                parser.add_option(action='callback',
+                              callback=self._optparse_value_callback(key),
+                                  help=self.registry.get_help(key),
+                                  *option_strings)
+
+    def _optparse_value_callback(self, cb_value):
+        def cb(option, opt, value, parser):
+            setattr(parser.values, self.name, self.type(cb_value))
+        return cb
+
+    def iter_switches(self):
+        """Iterate through the list of switches provided by the option
+
+        :return: an iterator of (name, short_name, argname, help)
+        """
+        for value in Option.iter_switches(self):
+            yield value
+        if self.value_switches:
+            for key in sorted(self.registry.keys()):
+                yield key, None, None, self.registry.get_help(key)
+
+
 class OptionParser(optparse.OptionParser):
     """OptionParser that raises exceptions instead of exiting"""
 
@@ -265,3 +347,21 @@
 _global_option('dry-run',
                help="show what would be done, but don't actually do anything")
 _global_option('name-from-revision', help='The path name in the old tree.')
+
+
+# prior to 0.14 these were always globally registered; the old dict is
+# available for plugins that use it but it should not be used.
+Option.SHORT_OPTIONS = symbol_versioning.DeprecatedDict(
+    symbol_versioning.zero_fourteen,
+    'SHORT_OPTIONS',
+    {
+        'F': Option.OPTIONS['file'],
+        'h': Option.OPTIONS['help'],
+        'm': Option.OPTIONS['message'],
+        'r': Option.OPTIONS['revision'],
+        'v': Option.OPTIONS['verbose'],
+        'l': Option.OPTIONS['long'],
+        'q': Option.OPTIONS['quiet'],
+    },
+    'Set the short option name when constructing the Option.',
+    )

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2007-01-05 06:32:57 +0000
+++ bzrlib/repository.py	2007-01-17 15:37:08 +0000
@@ -97,6 +97,7 @@
 
         returns the sha1 of the serialized inventory.
         """
+        _mod_revision.check_not_reserved_id(revid)
         assert inv.revision_id is None or inv.revision_id == revid, \
             "Mismatch between inventory revision" \
             " id and insertion revid (%r, %r)" % (inv.revision_id, revid)
@@ -128,6 +129,7 @@
                        If supplied its signature_needed method will be used
                        to determine if a signature should be made.
         """
+        _mod_revision.check_not_reserved_id(rev_id)
         if config is not None and config.signature_needed():
             if inv is None:
                 inv = self.get_inventory(rev_id)

=== modified file 'bzrlib/revision.py'
--- bzrlib/revision.py	2006-12-15 18:16:34 +0000
+++ bzrlib/revision.py	2007-01-17 15:37:08 +0000
@@ -454,3 +454,17 @@
         next = best_ancestor(next)
     path.reverse()
     return path
+
+
+def is_reserved_id(revision_id):
+    """Determine whether a revision id is reserved
+
+    :return: True if the revision is is reserved, False otherwise
+    """
+    return isinstance(revision_id, basestring) and revision_id.endswith(':')
+
+
+def check_not_reserved_id(revision_id):
+    """Raise ReservedId if the supplied revision_id is reserved"""
+    if is_reserved_id(revision_id):
+        raise errors.ReservedId(revision_id)

=== modified file 'bzrlib/symbol_versioning.py'
--- bzrlib/symbol_versioning.py	2006-11-27 23:11:54 +0000
+++ bzrlib/symbol_versioning.py	2007-01-16 15:56:53 +0000
@@ -1,5 +1,5 @@
-# Copyright (C) 2006 Canonical Ltd
-#   Authors: Robert Collins <robert.collins@canonical.com>
+# Copyright (C) 2006, 2007 Canonical Ltd
+#   Authors: Robert Collins <robert.collins@canonical.com> and others
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -33,6 +33,7 @@
            'zero_twelve',
            'zero_thirteen',
            'zero_fourteen',
+           'zero_fifteen',
            ]
 
 from warnings import warn
@@ -47,6 +48,7 @@
 zero_twelve = "%s was deprecated in version 0.12."
 zero_thirteen = "%s was deprecated in version 0.13."
 zero_fourteen = "%s was deprecated in version 0.14."
+zero_fifteen = "%s was deprecated in version 0.15."
 
 
 def set_warning_method(method):
@@ -176,6 +178,52 @@
     decorated_callable.is_deprecated = True
 
 
+def _dict_deprecation_wrapper(wrapped_method):
+    """Returns a closure that emits a warning and calls the superclass"""
+    def cb(dep_dict, *args, **kwargs):
+        msg = 'access to %s' % (dep_dict._variable_name, )
+        msg = dep_dict._deprecation_version % (msg,)
+        if dep_dict._advice:
+            msg += ' ' + dep_dict._advice
+        warn(msg, DeprecationWarning, stacklevel=2)
+        return wrapped_method(dep_dict, *args, **kwargs)
+    return cb
+
+
+class DeprecatedDict(dict):
+    """A dictionary that complains when read or written."""
+
+    is_deprecated = True
+
+    def __init__(self,
+        deprecation_version,
+        variable_name,
+        initial_value,
+        advice,
+        ):
+        """Create a dict that warns when read or modified.
+
+        :param deprecation_version: something like zero_nine
+        :param initial_value: The contents of the dict
+        :param variable_name: This allows better warnings to be printed
+        :param advice: String of advice on what callers should do instead 
+            of using this variable.
+        """
+        self._deprecation_version = deprecation_version
+        self._variable_name = variable_name
+        self._advice = advice
+        dict.__init__(self, initial_value)
+
+    # This isn't every possible method but it should trap anyone using the
+    # dict -- add more if desired
+    __len__ = _dict_deprecation_wrapper(dict.__len__)
+    __getitem__ = _dict_deprecation_wrapper(dict.__getitem__)
+    __setitem__ = _dict_deprecation_wrapper(dict.__setitem__)
+    __delitem__ = _dict_deprecation_wrapper(dict.__delitem__)
+    keys = _dict_deprecation_wrapper(dict.keys)
+    __contains__ = _dict_deprecation_wrapper(dict.__contains__)
+
+
 def deprecated_list(deprecation_version, variable_name,
                     initial_value, extra=None):
     """Create a list that warns when modified

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2007-01-11 08:45:51 +0000
+++ bzrlib/tests/__init__.py	2007-01-19 19:42:19 +0000
@@ -661,9 +661,19 @@
                 excName = str(excClass)
             raise self.failureException, "%s not raised" % excName
 
-    def assertIs(self, left, right):
+    def assertIs(self, left, right, message=None):
         if not (left is right):
-            raise AssertionError("%r is not %r." % (left, right))
+            if message is not None:
+                raise AssertionError(message)
+            else:
+                raise AssertionError("%r is not %r." % (left, right))
+
+    def assertIsNot(self, left, right, message=None):
+        if (left is right):
+            if message is not None:
+                raise AssertionError(message)
+            else:
+                raise AssertionError("%r is %r." % (left, right))
 
     def assertTransportMode(self, transport, path, mode):
         """Fail if a path does not have mode mode.
@@ -1539,11 +1549,11 @@
 
     def failUnlessExists(self, path):
         """Fail unless path, which may be abs or relative, exists."""
-        self.failUnless(osutils.lexists(path))
+        self.failUnless(osutils.lexists(path),path+" does not exist")
 
     def failIfExists(self, path):
         """Fail if path, which may be abs or relative, exists."""
-        self.failIf(osutils.lexists(path))
+        self.failIf(osutils.lexists(path),path+" exists")
 
 
 class TestCaseWithTransport(TestCaseInTempDir):
@@ -1658,9 +1668,22 @@
     return result
 
 
+def sort_suite_by_re(suite, pattern):
+    first = []
+    second = []
+    filter_re = re.compile(pattern)
+    for test in iter_suite_tests(suite):
+        if filter_re.search(test.id()):
+            first.append(test)
+        else:
+            second.append(test)
+    return TestUtil.TestSuite(first + second)
+
+
 def run_suite(suite, name='test', verbose=False, pattern=".*",
               stop_on_failure=False, keep_output=False,
-              transport=None, lsprof_timed=None, bench_history=None):
+              transport=None, lsprof_timed=None, bench_history=None,
+              matching_tests_first=None):
     TestCase._gather_lsprof_in_benchmarks = lsprof_timed
     if verbose:
         verbosity = 2
@@ -1673,7 +1696,10 @@
                             bench_history=bench_history)
     runner.stop_on_failure=stop_on_failure
     if pattern != '.*':
-        suite = filter_suite_by_re(suite, pattern)
+        if matching_tests_first:
+            suite = sort_suite_by_re(suite, pattern)
+        else:
+            suite = filter_suite_by_re(suite, pattern)
     result = runner.run(suite)
     return result.wasSuccessful()
 
@@ -1683,7 +1709,8 @@
              transport=None,
              test_suite_factory=None,
              lsprof_timed=None,
-             bench_history=None):
+             bench_history=None,
+             matching_tests_first=None):
     """Run the whole test suite under the enhanced runner"""
     # XXX: Very ugly way to do this...
     # Disable warning about old formats because we don't want it to disturb
@@ -1705,7 +1732,8 @@
                      stop_on_failure=stop_on_failure, keep_output=keep_output,
                      transport=transport,
                      lsprof_timed=lsprof_timed,
-                     bench_history=bench_history)
+                     bench_history=bench_history,
+                     matching_tests_first=matching_tests_first)
     finally:
         default_transport = old_transport
 

=== modified file 'bzrlib/tests/blackbox/test_ls.py'
--- bzrlib/tests/blackbox/test_ls.py	2007-01-02 15:58:27 +0000
+++ bzrlib/tests/blackbox/test_ls.py	2007-01-02 20:15:07 +0000
@@ -173,6 +173,20 @@
         os.chdir('subdir')
         self.ls_equals('', '--revision', '1')
 
+    def test_ls_branch(self):
+        """If a branch is specified, files are listed from it"""
+        self.build_tree(['subdir/', 'subdir/b'])
+        self.wt.add(['subdir', 'subdir/b'])
+        self.wt.commit('committing')
+        branch = self.make_branch('branchdir')
+        branch.pull(self.wt.branch)
+        self.ls_equals('branchdir/subdir\n'
+                       'branchdir/subdir/b\n',
+                       'branchdir')
+        self.ls_equals('branchdir/subdir\n'
+                       'branchdir/subdir/b\n',
+                       'branchdir', '--revision', '1')
+
     def test_ls_ignored(self):
         # Now try to do ignored files.
         self.wt.add(['a', '.bzrignore'])

=== modified file 'bzrlib/tests/blackbox/test_missing.py'
--- bzrlib/tests/blackbox/test_missing.py	2006-10-11 23:08:27 +0000
+++ bzrlib/tests/blackbox/test_missing.py	2007-01-24 13:03:32 +0000
@@ -20,11 +20,13 @@
 
 import os
 
+from bzrlib import osutils
+
 from bzrlib.branch import Branch
-from bzrlib.tests import TestCaseInTempDir
-
-
-class TestMissing(TestCaseInTempDir):
+from bzrlib.tests import TestCaseWithTransport
+
+
+class TestMissing(TestCaseWithTransport):
 
     def test_missing(self):
         def bzr(*args, **kwargs):
@@ -118,3 +120,26 @@
         self.assertEqual("Branches are up to date.\n", 
                          bzr('missing', '../a'))
 
+    def test_missing_check_last_location(self):
+        # check that last location shown as filepath not file URL
+
+        # create a source branch
+        os.mkdir('a')
+        os.chdir('a')
+        wt = self.make_branch_and_tree('.')
+        b = wt.branch
+        self.build_tree(['foo'])
+        wt.add('foo')
+        wt.commit('initial')
+
+        location = osutils.getcwd() + '/'
+
+        # clone
+        b.bzrdir.sprout('../b')
+
+        # check last location
+        lines, err = self.run_bzr('missing', working_dir='../b')
+        self.assertEquals('Using last location: %s\n'
+                          'Branches are up to date.\n' % location,
+                          lines)
+        self.assertEquals('', err)

=== modified file 'bzrlib/tests/blackbox/test_mv.py'
--- bzrlib/tests/blackbox/test_mv.py	2006-12-13 09:31:35 +0000
+++ bzrlib/tests/blackbox/test_mv.py	2007-01-20 01:44:46 +0000
@@ -22,6 +22,7 @@
     osutils,
     workingtree,
     )
+
 from bzrlib.tests import (
     TestCaseWithTransport,
     TestSkipped,
@@ -30,6 +31,23 @@
 
 class TestMove(TestCaseWithTransport):
 
+    def assertInWorkingTree(self,path):
+        tree = workingtree.WorkingTree.open('.')
+        self.assertIsNot(tree.path2id(path), None,
+            path+' not in working tree.')
+
+    def assertNotInWorkingTree(self,path):
+        tree = workingtree.WorkingTree.open('.')
+        self.assertIs(tree.path2id(path), None, path+' in working tree.')
+
+    def assertMoved(self,from_path,to_path):
+        """Assert that to_path is existing and versioned but from_path not. """
+        self.failIfExists(from_path)
+        self.assertNotInWorkingTree(from_path)
+
+        self.failUnlessExists(to_path)
+        self.assertInWorkingTree(to_path)
+
     def test_mv_modes(self):
         """Test two modes of operation for mv"""
         tree = self.make_branch_and_tree('.')
@@ -37,36 +55,32 @@
         tree.add(['a', 'c', 'subdir'])
 
         self.run_bzr('mv', 'a', 'b')
-        self.failUnlessExists('b')
-        self.failIfExists('a')
+        self.assertMoved('a','b')
 
         self.run_bzr('mv', 'b', 'subdir')
-        self.failUnlessExists('subdir/b')
-        self.failIfExists('b')
+        self.assertMoved('b','subdir/b')
 
         self.run_bzr('mv', 'subdir/b', 'a')
-        self.failUnlessExists('a')
-        self.failIfExists('subdir/b')
+        self.assertMoved('subdir/b','a')
 
         self.run_bzr('mv', 'a', 'c', 'subdir')
-        self.failUnlessExists('subdir/a')
-        self.failUnlessExists('subdir/c')
-        self.failIfExists('a')
-        self.failIfExists('c')
+        self.assertMoved('a','subdir/a')
+        self.assertMoved('c','subdir/c')
 
         self.run_bzr('mv', 'subdir/a', 'subdir/newa')
-        self.failUnlessExists('subdir/newa')
-        self.failIfExists('subdir/a')
+        self.assertMoved('subdir/a','subdir/newa')
 
     def test_mv_unversioned(self):
         self.build_tree(['unversioned.txt'])
         self.run_bzr_error(
-            ["^bzr: ERROR: can't rename: old name .* is not versioned$"],
+            ["^bzr: ERROR: Could not rename unversioned.txt => elsewhere."
+             " .*unversioned.txt is not versioned$"],
             'mv', 'unversioned.txt', 'elsewhere')
 
     def test_mv_nonexisting(self):
         self.run_bzr_error(
-            ["^bzr: ERROR: can't rename: old working file .* does not exist$"],
+            ["^bzr: ERROR: Could not rename doesnotexist => somewhereelse."
+             " .*doesnotexist is not versioned$"],
             'mv', 'doesnotexist', 'somewhereelse')
 
     def test_mv_unqualified(self):
@@ -78,11 +92,12 @@
         tree.add(['test.txt'])
 
         self.run_bzr_error(
-            ["^bzr: ERROR: destination u'sub1' is not a versioned directory$"],
+            ["^bzr: ERROR: Could not move to sub1: sub1 is not versioned$"],
             'mv', 'test.txt', 'sub1')
-        
+
         self.run_bzr_error(
-            ["^bzr: ERROR: can't determine destination directory id for u'sub1'$"],
+            ["^bzr: ERROR: Could not move test.txt => .*hello.txt: "
+             "sub1 is not versioned$"],
             'mv', 'test.txt', 'sub1/hello.txt')
         
     def test_mv_dirs(self):
@@ -91,40 +106,35 @@
         tree.add(['hello.txt', 'sub1'])
 
         self.run_bzr('mv', 'sub1', 'sub2')
-        self.failUnlessExists('sub2')
-        self.failIfExists('sub1')
+        self.assertMoved('sub1','sub2')
+
         self.run_bzr('mv', 'hello.txt', 'sub2')
-        self.failUnlessExists("sub2/hello.txt")
-        self.failIfExists("hello.txt")
+        self.assertMoved('hello.txt','sub2/hello.txt')
 
         tree.read_working_inventory()
-        tree.commit('commit with some things moved to subdirs')
 
         self.build_tree(['sub1/'])
         tree.add(['sub1'])
         self.run_bzr('mv', 'sub2/hello.txt', 'sub1')
-        self.failIfExists('sub2/hello.txt')
-        self.failUnlessExists('sub1/hello.txt')
+        self.assertMoved('sub2/hello.txt','sub1/hello.txt')
+
         self.run_bzr('mv', 'sub2', 'sub1')
-        self.failIfExists('sub2')
-        self.failUnlessExists('sub1/sub2')
+        self.assertMoved('sub2','sub1/sub2')
 
     def test_mv_relative(self):
         self.build_tree(['sub1/', 'sub1/sub2/', 'sub1/hello.txt'])
         tree = self.make_branch_and_tree('.')
         tree.add(['sub1', 'sub1/sub2', 'sub1/hello.txt'])
-        tree.commit('initial tree')
 
         os.chdir('sub1/sub2')
         self.run_bzr('mv', '../hello.txt', '.')
         self.failUnlessExists('./hello.txt')
         tree.read_working_inventory()
-        tree.commit('move to parent directory')
 
         os.chdir('..')
-
         self.run_bzr('mv', 'sub2/hello.txt', '.')
-        self.failUnlessExists('hello.txt')
+        os.chdir('..')
+        self.assertMoved('sub1/sub2/hello.txt','sub1/hello.txt')
 
     def test_mv_smoke_aliases(self):
         # just test that aliases for mv exist, if their behaviour is changed in
@@ -147,3 +157,215 @@
         self.run_bzr('mv', 'c/b', 'b')
         tree = workingtree.WorkingTree.open('.')
         self.assertEqual('b-id', tree.path2id('b'))
+
+    def test_mv_already_moved_file(self):
+        """Test bzr mv original_file to moved_file.
+
+        Tests if a file which has allready been moved by an external tool,
+        is handled correctly by bzr mv.
+        Setup: a is in the working tree, b does not exist.
+        User does: mv a b; bzr mv a b
+        """
+        self.build_tree(['a'])
+        tree = self.make_branch_and_tree('.')
+        tree.add(['a'])
+
+        os.rename('a', 'b')
+        self.run_bzr('mv', 'a', 'b')
+        self.assertMoved('a','b')
+
+    def test_mv_already_moved_file_to_versioned_target(self):
+        """Test bzr mv existing_file to versioned_file.
+
+        Tests if an attempt to move an existing versioned file
+        to another versiond file will fail.
+        Setup: a and b are in the working tree.
+        User does: rm b; mv a b; bzr mv a b
+        """
+        self.build_tree(['a', 'b'])
+        tree = self.make_branch_and_tree('.')
+        tree.add(['a', 'b'])
+
+        os.remove('b')
+        os.rename('a', 'b')
+        self.run_bzr_error(
+            ["^bzr: ERROR: Could not move a => b. b is already versioned$"],
+            'mv', 'a', 'b')
+        #check that nothing changed
+        self.failIfExists('a')
+        self.failUnlessExists('b')
+
+    def test_mv_already_moved_file_into_subdir(self):
+        """Test bzr mv original_file to versioned_directory/file.
+
+        Tests if a file which has already been moved into a versioned
+        directory by an external tool, is handled correctly by bzr mv.
+        Setup: a and sub/ are in the working tree.
+        User does: mv a sub/a; bzr mv a sub/a
+        """
+        self.build_tree(['a', 'sub/'])
+        tree = self.make_branch_and_tree('.')
+        tree.add(['a', 'sub'])
+
+        os.rename('a', 'sub/a')
+        self.run_bzr('mv', 'a', 'sub/a')
+        self.assertMoved('a','sub/a')
+
+    def test_mv_already_moved_file_into_unversioned_subdir(self):
+        """Test bzr mv original_file to unversioned_directory/file.
+
+        Tests if an attempt to move an existing versioned file
+        into an unversioned directory will fail.
+        Setup: a is in the working tree, sub/ is not.
+        User does: mv a sub/a; bzr mv a sub/a
+        """
+        self.build_tree(['a', 'sub/'])
+        tree = self.make_branch_and_tree('.')
+        tree.add(['a'])
+
+        os.rename('a', 'sub/a')
+        self.run_bzr_error(
+            ["^bzr: ERROR: Could not move a => a: sub is not versioned$"],
+            'mv', 'a', 'sub/a')
+        self.failIfExists('a')
+        self.failUnlessExists('sub/a')
+
+    def test_mv_already_moved_files_into_subdir(self):
+        """Test bzr mv original_files to versioned_directory.
+
+        Tests if files which has already been moved into a versioned
+        directory by an external tool, is handled correctly by bzr mv.
+        Setup: a1, a2, sub are in the working tree.
+        User does: mv a1 sub/.; bzr mv a1 a2 sub
+        """
+        self.build_tree(['a1', 'a2', 'sub/'])
+        tree = self.make_branch_and_tree('.')
+        tree.add(['a1', 'a2', 'sub'])
+
+        os.rename('a1', 'sub/a1')
+        self.run_bzr('mv', 'a1', 'a2', 'sub')
+        self.assertMoved('a1','sub/a1')
+        self.assertMoved('a2','sub/a2')
+
+    def test_mv_already_moved_files_into_unversioned_subdir(self):
+        """Test bzr mv original_file to unversioned_directory.
+
+        Tests if an attempt to move existing versioned file
+        into an unversioned directory will fail.
+        Setup: a1, a2 are in the working tree, sub is not.
+        User does: mv a1 sub/.; bzr mv a1 a2 sub
+        """
+        self.build_tree(['a1', 'a2', 'sub/'])
+        tree = self.make_branch_and_tree('.')
+        tree.add(['a1', 'a2'])
+
+        os.rename('a1', 'sub/a1')
+        self.run_bzr_error(
+            ["^bzr: ERROR: Could not move to sub. sub is not versioned$"],
+            'mv', 'a1', 'a2', 'sub')
+        self.failIfExists('a1')
+        self.failUnlessExists('sub/a1')
+        self.failUnlessExists('a2')
+        self.failIfExists('sub/a2')
+
+    def test_mv_already_moved_file_forcing_after(self):
+        """Test bzr mv versioned_file to unversioned_file.
+
+        Tests if an attempt to move an existing versioned file to an existing
+        unversioned file will fail, informing the user to use the --after
+        option to force this.
+        Setup: a is in the working tree, b not versioned.
+        User does: mv a b; touch a; bzr mv a b
+        """
+        self.build_tree(['a', 'b'])
+        tree = self.make_branch_and_tree('.')
+        tree.add(['a'])
+
+        os.rename('a', 'b')
+        self.build_tree(['a']) #touch a
+        self.run_bzr_error(
+            ["^bzr: ERROR: Could not rename a => b because both files exist."
+             " \(Use --after to update the Bazaar id\)$"],
+            'mv', 'a', 'b')
+        self.failUnlessExists('a')
+        self.failUnlessExists('b')
+
+    def test_mv_already_moved_file_using_after(self):
+        """Test bzr mv --after versioned_file to unversioned_file.
+
+        Tests if an existing versioned file can be forced to move to an
+        existing unversioned file using the --after option. With the result
+        that bazaar considers the unversioned_file to be moved from
+        versioned_file and versioned_file will become unversioned.
+        Setup: a is in the working tree and b exists.
+        User does: mv a b; touch a; bzr mv a b --after
+        Resulting in a => b and a is unknown.
+        """
+        self.build_tree(['a', 'b'])
+        tree = self.make_branch_and_tree('.')
+        tree.add(['a'])
+        os.rename('a', 'b')
+        self.build_tree(['a']) #touch a
+
+        self.run_bzr('mv', 'a', 'b', '--after')
+        self.failUnlessExists('a')
+        self.assertNotInWorkingTree('a')#a should be unknown now.
+        self.failUnlessExists('b')
+        self.assertInWorkingTree('b')
+
+    def test_mv_already_moved_files_forcing_after(self):
+        """Test bzr mv versioned_files to directory/unversioned_file.
+
+        Tests if an attempt to move an existing versioned file to an existing
+        unversioned file in some other directory will fail, informing the user
+        to use the --after option to force this.
+
+        Setup: a1, a2, sub are versioned and in the working tree,
+               sub/a1, sub/a2 are in working tree.
+        User does: mv a* sub; touch a1; touch a2; bzr mv a1 a2 sub
+        """
+        self.build_tree(['a1', 'a2', 'sub/', 'sub/a1', 'sub/a2'])
+        tree = self.make_branch_and_tree('.')
+        tree.add(['a1', 'a2', 'sub'])
+        os.rename('a1', 'sub/a1')
+        os.rename('a2', 'sub/a2')
+        self.build_tree(['a1']) #touch a1
+        self.build_tree(['a2']) #touch a2
+
+        self.run_bzr_error(
+            ["^bzr: ERROR: Could not rename a1 => sub/a1 because both files exist."
+             " \(Use --after to update the Bazaar id\)$"],
+            'mv', 'a1', 'a2', 'sub')
+        self.failUnlessExists('a1')
+        self.failUnlessExists('a2')
+        self.failUnlessExists('sub/a1')
+        self.failUnlessExists('sub/a2')
+
+    def test_mv_already_moved_files_using_after(self):
+        """Test bzr mv --after versioned_file to directory/unversioned_file.
+
+        Tests if an existing versioned file can be forced to move to an
+        existing unversioned file in some other directory using the --after
+        option. With the result that bazaar considers
+        directory/unversioned_file to be moved from versioned_file and
+        versioned_file will become unversioned.
+
+        Setup: a1, a2, sub are versioned and in the working tree,
+               sub/a1, sub/a2 are in working tree.
+        User does: mv a* sub; touch a1; touch a2; bzr mv a1 a2 sub --after
+        """
+        self.build_tree(['a1', 'a2', 'sub/', 'sub/a1', 'sub/a2'])
+        tree = self.make_branch_and_tree('.')
+        tree.add(['a1', 'a2', 'sub'])
+        os.rename('a1', 'sub/a1')
+        os.rename('a2', 'sub/a2')
+        self.build_tree(['a1']) #touch a1
+        self.build_tree(['a2']) #touch a2
+
+        self.run_bzr('mv', 'a1', 'a2', 'sub', '--after')
+        self.failUnlessExists('a1')
+        self.failUnlessExists('a2')
+        self.failUnlessExists('sub/a1')
+        self.failUnlessExists('sub/a2')
+        self.assertInWorkingTree('sub/a1')
+        self.assertInWorkingTree('sub/a2')

=== modified file 'bzrlib/tests/branch_implementations/test_branch.py'
--- bzrlib/tests/branch_implementations/test_branch.py	2006-12-07 16:21:39 +0000
+++ bzrlib/tests/branch_implementations/test_branch.py	2007-01-11 17:20:25 +0000
@@ -87,6 +87,7 @@
         self.assertEquals(br.revision_history(), ["rev1",])
         br.append_revision("rev2", "rev3")
         self.assertEquals(br.revision_history(), ["rev1", "rev2", "rev3"])
+        self.assertRaises(errors.ReservedId, br.append_revision, 'current:')
 
     def test_fetch_revisions(self):
         """Test fetch-revision operation."""

=== modified file 'bzrlib/tests/repository_implementations/test_repository.py'
--- bzrlib/tests/repository_implementations/test_repository.py	2007-01-05 17:12:03 +0000
+++ bzrlib/tests/repository_implementations/test_repository.py	2007-01-17 15:37:08 +0000
@@ -516,6 +516,13 @@
         self.assertEqual({}, graph.get_ancestors())
         self.assertEqual({}, graph.get_descendants())
 
+    def test_reserved_id(self):
+        repo = self.make_repository('repository')
+        self.assertRaises(errors.ReservedId, repo.add_inventory, 'reserved:',
+                          None, None)
+        self.assertRaises(errors.ReservedId, repo.add_revision, 'reserved:',
+                          None)
+
 
 class TestCaseWithCorruptRepository(TestCaseWithRepository):
 

=== modified file 'bzrlib/tests/test_bzrdir.py'
--- bzrlib/tests/test_bzrdir.py	2007-01-05 17:12:03 +0000
+++ bzrlib/tests/test_bzrdir.py	2007-01-17 14:36:49 +0000
@@ -19,11 +19,13 @@
 For interface contract tests, see tests/bzr_dir_implementations.
 """
 
+import os.path
 from StringIO import StringIO
 
 from bzrlib import (
     help_topics,
     symbol_versioning,
+    urlutils,
     )
 import bzrlib.branch
 import bzrlib.bzrdir as bzrdir
@@ -33,7 +35,7 @@
                            UnsupportedFormatError,
                            )
 import bzrlib.repository as repository
-from bzrlib.tests import TestCase, TestCaseWithTransport
+from bzrlib.tests import TestCase, TestCaseWithTransport, test_sftp_transport
 from bzrlib.tests.HttpServer import HttpServer
 from bzrlib.transport import get_transport
 from bzrlib.transport.memory import MemoryServer
@@ -422,6 +424,28 @@
             get_transport(self.get_readonly_url('g/p/q')))
         self.assertEqual('g/p/q', relpath)
 
+    def test_open_containing_tree_or_branch(self):
+        def local_branch_path(branch):
+             return os.path.realpath(
+                urlutils.local_path_from_url(branch.base))
+
+        self.make_branch_and_tree('topdir')
+        tree, branch, relpath = bzrdir.BzrDir.open_containing_tree_or_branch(
+            'topdir/foo')
+        self.assertEqual(os.path.realpath('topdir'),
+                         os.path.realpath(tree.basedir))
+        self.assertEqual(os.path.realpath('topdir'),
+                         local_branch_path(branch))
+        self.assertIs(tree.bzrdir, branch.bzrdir)
+        self.assertEqual('foo', relpath)
+        self.make_branch('topdir/foo')
+        tree, branch, relpath = bzrdir.BzrDir.open_containing_tree_or_branch(
+            'topdir/foo')
+        self.assertIs(tree, None)
+        self.assertEqual(os.path.realpath('topdir/foo'),
+                         local_branch_path(branch))
+        self.assertEqual('', relpath)
+
     def test_open_from_transport(self):
         # transport pointing at bzrdir should give a bzrdir with root transport
         # set to the given transport
@@ -645,3 +669,9 @@
         result.open_branch()
         result.open_repository()
 
+
+class TestRemoteSFTP(test_sftp_transport.TestCaseWithSFTPServer):
+
+    def test_open_containing_tree_or_branch(self):
+        tree = self.make_branch_and_tree('tree')
+        bzrdir.BzrDir.open_containing_tree_or_branch(self.get_url('tree'))

=== modified file 'bzrlib/tests/test_inv.py'
--- bzrlib/tests/test_inv.py	2006-12-12 22:40:01 +0000
+++ bzrlib/tests/test_inv.py	2006-12-22 17:11:36 +0000
@@ -139,6 +139,51 @@
             ('src/sub/a', 'a-id'),
             ], [(path, ie.file_id) for path, ie in inv.iter_entries_by_dir()])
             
+        self.assertEqual([
+            ('', ROOT_ID),
+            ('Makefile', 'makefile-id'),
+            ('doc', 'doc-id'),
+            ('src', 'src-id'),
+            ('zz', 'zz-id'),
+            ('src/bye.c', 'bye-id'),
+            ('src/hello.c', 'hello-id'),
+            ('src/sub', 'sub-id'),
+            ('src/zz.c', 'zzc-id'),
+            ('src/sub/a', 'a-id'),
+            ], [(path, ie.file_id) for path, ie in inv.iter_entries_by_dir(
+                specific_file_ids=('a-id', 'zzc-id', 'doc-id', ROOT_ID,
+                'hello-id', 'bye-id', 'zz-id', 'src-id', 'makefile-id', 
+                'sub-id'))])
+
+        self.assertEqual([
+            ('Makefile', 'makefile-id'),
+            ('doc', 'doc-id'),
+            ('zz', 'zz-id'),
+            ('src/bye.c', 'bye-id'),
+            ('src/hello.c', 'hello-id'),
+            ('src/zz.c', 'zzc-id'),
+            ('src/sub/a', 'a-id'),
+            ], [(path, ie.file_id) for path, ie in inv.iter_entries_by_dir(
+                specific_file_ids=('a-id', 'zzc-id', 'doc-id',
+                'hello-id', 'bye-id', 'zz-id', 'makefile-id'))])
+
+        self.assertEqual([
+            ('Makefile', 'makefile-id'),
+            ('src/bye.c', 'bye-id'),
+            ], [(path, ie.file_id) for path, ie in inv.iter_entries_by_dir(
+                specific_file_ids=('bye-id', 'makefile-id'))])
+
+        self.assertEqual([
+            ('Makefile', 'makefile-id'),
+            ('src/bye.c', 'bye-id'),
+            ], [(path, ie.file_id) for path, ie in inv.iter_entries_by_dir(
+                specific_file_ids=('bye-id', 'makefile-id'))])
+
+        self.assertEqual([
+            ('src/bye.c', 'bye-id'),
+            ], [(path, ie.file_id) for path, ie in inv.iter_entries_by_dir(
+                specific_file_ids=('bye-id',))])
+
     def test_version(self):
         """Inventory remembers the text's version."""
         inv = Inventory()

=== modified file 'bzrlib/tests/test_options.py'
--- bzrlib/tests/test_options.py	2007-01-10 07:28:57 +0000
+++ bzrlib/tests/test_options.py	2007-01-17 16:32:40 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,14 +14,21 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+from bzrlib import (
+    builtins,
+    bzrdir,
+    errors,
+    option,
+    repository,
+    symbol_versioning,
+    )
 from bzrlib.builtins import cmd_commit, cmd_log, cmd_status
 from bzrlib.commands import Command, parse_args
-from bzrlib import errors
-from bzrlib import option
 from bzrlib.tests import TestCase
 
-# TODO: might be nice to just parse them into a structured form and test
-# against that, rather than running the whole command.
+def parse(options, args):
+    parser = option.get_optparser(dict((o.name, o) for o in options))
+    return parser.parse_args(args)
 
 class OptionTests(TestCase):
     """Command-line option tests"""
@@ -69,28 +76,106 @@
         force_opt = option.Option.OPTIONS['force']
         self.assertEquals(force_opt.short_name(), None)
 
+    def test_set_short_name(self):
+        o = option.Option('wiggle')
+        o.set_short_name('w')
+        self.assertEqual(o.short_name(), 'w')
+
+    def test_old_short_names(self):
+        # test the deprecated method for getting and setting short option
+        # names
+        expected_warning = (
+            "access to SHORT_OPTIONS was deprecated in version 0.14."
+            " Set the short option name when constructing the Option.",
+            DeprecationWarning, 2)
+        _warnings = []
+        def capture_warning(message, category, stacklevel=None):
+            _warnings.append((message, category, stacklevel))
+        old_warning_method = symbol_versioning.warn
+        try:
+            # an example of the kind of thing plugins might want to do through
+            # the old interface - make a new option and then give it a short
+            # name.
+            symbol_versioning.set_warning_method(capture_warning)
+            example_opt = option.Option('example', help='example option')
+            option.Option.SHORT_OPTIONS['w'] = example_opt
+            self.assertEqual(example_opt.short_name(), 'w')
+            self.assertEqual([expected_warning], _warnings)
+            # now check that it can actually be parsed with the registered
+            # value
+            opts, args = parse([example_opt], ['-w', 'foo'])
+            self.assertEqual(opts.example, True)
+            self.assertEqual(args, ['foo'])
+        finally:
+            symbol_versioning.set_warning_method(old_warning_method)
+
     def test_allow_dash(self):
         """Test that we can pass a plain '-' as an argument."""
         self.assertEqual((['-'], {}), parse_args(cmd_commit(), ['-']))
 
+    def parse(self, options, args):
+        parser = option.get_optparser(dict((o.name, o) for o in options))
+        return parser.parse_args(args)
+
     def test_conversion(self):
-        def parse(options, args):
-            parser = option.get_optparser(dict((o.name, o) for o in options))
-            return parser.parse_args(args)
         options = [option.Option('hello')]
-        opts, args = parse(options, ['--no-hello', '--hello'])
+        opts, args = self.parse(options, ['--no-hello', '--hello'])
         self.assertEqual(True, opts.hello)
-        opts, args = parse(options, [])
+        opts, args = self.parse(options, [])
         self.assertEqual(option.OptionParser.DEFAULT_VALUE, opts.hello)
-        opts, args = parse(options, ['--hello', '--no-hello'])
+        opts, args = self.parse(options, ['--hello', '--no-hello'])
         self.assertEqual(False, opts.hello)
         options = [option.Option('number', type=int)]
-        opts, args = parse(options, ['--number', '6'])
+        opts, args = self.parse(options, ['--number', '6'])
         self.assertEqual(6, opts.number)
-        self.assertRaises(errors.BzrCommandError, parse, options, ['--number'])
-        self.assertRaises(errors.BzrCommandError, parse, options, 
+        self.assertRaises(errors.BzrCommandError, self.parse, options,
+                          ['--number'])
+        self.assertRaises(errors.BzrCommandError, self.parse, options,
                           ['--no-number'])
 
+    def test_registry_conversion(self):
+        registry = bzrdir.BzrDirFormatRegistry()
+        registry.register_metadir('one', 'RepositoryFormat7', 'one help')
+        registry.register_metadir('two', 'RepositoryFormatKnit1', 'two help')
+        registry.set_default('one')
+        options = [option.RegistryOption('format', '', registry, str)]
+        opts, args = self.parse(options, ['--format', 'one'])
+        self.assertEqual({'format':'one'}, opts)
+        opts, args = self.parse(options, ['--format', 'two'])
+        self.assertEqual({'format':'two'}, opts)
+        self.assertRaises(errors.BadOptionValue, self.parse, options,
+                          ['--format', 'three'])
+        self.assertRaises(errors.BzrCommandError, self.parse, options,
+                          ['--two'])
+        options = [option.RegistryOption('format', '', registry, str,
+                   value_switches=True)]
+        opts, args = self.parse(options, ['--two'])
+        self.assertEqual({'format':'two'}, opts)
+        opts, args = self.parse(options, ['--two', '--one'])
+        self.assertEqual({'format':'one'}, opts)
+        opts, args = self.parse(options, ['--two', '--one',
+                                          '--format', 'two'])
+        self.assertEqual({'format':'two'}, opts)
+
+    def test_registry_converter(self):
+        options = [option.RegistryOption('format', '',
+                   bzrdir.format_registry, builtins.get_format_type)]
+        opts, args = self.parse(options, ['--format', 'knit'])
+        self.assertIsInstance(opts.format.repository_format,
+                              repository.RepositoryFormatKnit1)
+
+    def test_help(self):
+        registry = bzrdir.BzrDirFormatRegistry()
+        registry.register_metadir('one', 'RepositoryFormat7', 'one help')
+        registry.register_metadir('two', 'RepositoryFormatKnit1', 'two help')
+        registry.set_default('one')
+        options = [option.RegistryOption('format', 'format help', registry,
+                   str, value_switches=True)]
+        parser = option.get_optparser(dict((o.name, o) for o in options))
+        value = parser.format_option_help()
+        self.assertContainsRe(value, 'format.*format help')
+        self.assertContainsRe(value, 'one.*one help')
+
     def test_iter_switches(self):
         opt = option.Option('hello', help='fg')
         self.assertEqual(list(opt.iter_switches()),
@@ -101,6 +186,22 @@
         opt = option.Option('hello', help='fg', type=int, argname='gar')
         self.assertEqual(list(opt.iter_switches()),
                          [('hello', None, 'GAR', 'fg')])
+        registry = bzrdir.BzrDirFormatRegistry()
+        registry.register_metadir('one', 'RepositoryFormat7', 'one help')
+        registry.register_metadir('two', 'RepositoryFormatKnit1', 'two help')
+        registry.set_default('one')
+        opt = option.RegistryOption('format', 'format help', registry,
+                                    value_switches=False)
+        self.assertEqual(list(opt.iter_switches()),
+                         [('format', None, 'ARG', 'format help')])
+        opt = option.RegistryOption('format', 'format help', registry,
+                                    value_switches=True)
+        self.assertEqual(list(opt.iter_switches()),
+                         [('format', None, 'ARG', 'format help'),
+                          ('default', None, None, 'one help'),
+                          ('one', None, None, 'one help'),
+                          ('two', None, None, 'two help'),
+                          ])
 
 #     >>> parse_args('log -r 500'.split())
 #     (['log'], {'revision': [<RevisionSpec_int 500>]})

=== modified file 'bzrlib/tests/test_revision.py'
--- bzrlib/tests/test_revision.py	2006-10-05 05:37:25 +0000
+++ bzrlib/tests/test_revision.py	2007-01-17 15:37:08 +0000
@@ -296,3 +296,16 @@
         self.assertEqual({'B':['A'],
                           'A':[]},
                          source.get_revision_graph('B'))
+
+
+class TestReservedId(TestCase):
+
+    def test_is_reserved_id(self):
+        self.assertEqual(True, revision.is_reserved_id(NULL_REVISION))
+        self.assertEqual(True, revision.is_reserved_id(
+            revision.CURRENT_REVISION))
+        self.assertEqual(True, revision.is_reserved_id('arch:'))
+        self.assertEqual(False, revision.is_reserved_id('null'))
+        self.assertEqual(False, revision.is_reserved_id(
+            'arch:a@example.com/c--b--v--r'))
+        self.assertEqual(False, revision.is_reserved_id(None))

=== modified file 'bzrlib/tests/test_symbol_versioning.py'
--- bzrlib/tests/test_symbol_versioning.py	2006-10-11 23:08:27 +0000
+++ bzrlib/tests/test_symbol_versioning.py	2007-01-11 08:08:21 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006, 2007 Canonical Ltd
 #   Authors: Robert Collins <robert.collins@canonical.com>
 #
 # This program is free software; you can redistribute it and/or modify
@@ -31,6 +31,14 @@
     'a_deprecated_list', ['one'], extra="Don't use me")
 
 
+a_deprecated_dict = symbol_versioning.DeprecatedDict(
+    symbol_versioning.zero_fourteen,
+    'a_deprecated_dict',
+    dict(a=42),
+    advice='Pull the other one!',
+    )
+
+
 class TestDeprecationWarnings(TestCase):
 
     def capture_warning(self, message, category, stacklevel=None):
@@ -81,8 +89,6 @@
         expected_warning = (
             "Modifying a_deprecated_list was deprecated in version 0.9."
             " Don't use me", DeprecationWarning, 3)
-        expected_doctstring = ('appending to a_deprecated_list is deprecated')
-
         old_warning_method = symbol_versioning.warn
         try:
             symbol_versioning.set_warning_method(self.capture_warning)
@@ -117,6 +123,26 @@
         finally:
             symbol_versioning.set_warning_method(old_warning_method)
 
+    def test_deprecated_dict(self):
+        expected_warning = (
+            "access to a_deprecated_dict was deprecated in version 0.14."
+            " Pull the other one!", DeprecationWarning, 2)
+        old_warning_method = symbol_versioning.warn
+        try:
+            symbol_versioning.set_warning_method(self.capture_warning)
+            self.assertEqual(len(a_deprecated_dict), 1)
+            self.assertEqual([expected_warning], self._warnings)
+
+            a_deprecated_dict['b'] = 42
+            self.assertEqual(a_deprecated_dict['b'], 42)
+            self.assertTrue('b' in a_deprecated_dict)
+            del a_deprecated_dict['b']
+            self.assertFalse('b' in a_deprecated_dict)
+            self.assertEqual([expected_warning] * 6, self._warnings)
+        finally:
+            symbol_versioning.set_warning_method(old_warning_method)
+
+
     def check_deprecated_callable(self, expected_warning, expected_docstring,
                                   expected_name, expected_module,
                                   deprecated_callable):

=== modified file 'bzrlib/tests/test_versionedfile.py'
--- bzrlib/tests/test_versionedfile.py	2006-11-28 01:04:15 +0000
+++ bzrlib/tests/test_versionedfile.py	2007-01-11 17:20:25 +0000
@@ -140,6 +140,21 @@
         except NotImplementedError:
             pass
 
+    def test_add_reserved(self):
+        vf = self.get_file()
+        self.assertRaises(errors.ReservedId,
+            vf.add_lines, 'a:', [], ['a\n', 'b\n', 'c\n'])
+
+        self.assertRaises(errors.ReservedId,
+            vf.add_delta, 'a:', [], None, 'sha1', False, ((0, 0, 0, []),))
+
+    def test_get_reserved(self):
+        vf = self.get_file()
+        self.assertRaises(errors.ReservedId, vf.get_delta, 'b:')
+        self.assertRaises(errors.ReservedId, vf.get_texts, ['b:'])
+        self.assertRaises(errors.ReservedId, vf.get_lines, 'b:')
+        self.assertRaises(errors.ReservedId, vf.get_text, 'b:')
+
     def test_get_delta(self):
         f = self.get_file()
         sha1s = self._setup_for_deltas(f)

=== modified file 'bzrlib/tests/workingtree_implementations/test_workingtree.py'
--- bzrlib/tests/workingtree_implementations/test_workingtree.py	2006-09-10 19:04:48 +0000
+++ bzrlib/tests/workingtree_implementations/test_workingtree.py	2007-01-18 04:11:51 +0000
@@ -288,6 +288,7 @@
         wt.branch.set_revision_history(['A', 'B'])
         wt.set_last_revision('A')
         self.assertEqual(['A'], wt.get_parent_ids())
+        self.assertRaises(errors.ReservedId, wt.set_last_revision, 'A:')
 
     def test_set_last_revision_different_to_branch(self):
         # working tree formats from the meta-dir format and newer support
@@ -681,3 +682,50 @@
                 tree.add, [u'a\u030a'])
         finally:
             osutils.normalized_filename = orig
+
+    def test_move_deprecated_correct_call_named(self):
+        """tree.move has the deprecated parameter 'to_name'.
+        It has been replaced by 'to_dir' for consistency.
+        Test the new API using named parameter"""
+        self.build_tree(['a1', 'sub1/'])
+        tree = self.make_branch_and_tree('.')
+        tree.add(['a1', 'sub1'])
+        tree.commit('initial commit')
+        tree.move(['a1'], to_dir='sub1', after=False)
+
+    def test_move_deprecated_correct_call_unnamed(self):
+        """tree.move has the deprecated parameter 'to_name'.
+        It has been replaced by 'to_dir' for consistency.
+        Test the new API using unnamed parameter"""
+        self.build_tree(['a1', 'sub1/'])
+        tree = self.make_branch_and_tree('.')
+        tree.add(['a1', 'sub1'])
+        tree.commit('initial commit')
+        tree.move(['a1'], 'sub1', after=False)
+
+    def test_move_deprecated_wrong_call(self):
+        """tree.move has the deprecated parameter 'to_name'.
+        It has been replaced by 'to_dir' for consistency.
+        Test the new API using wrong parameter"""
+        self.build_tree(['a1', 'sub1/'])
+        tree = self.make_branch_and_tree('.')
+        tree.add(['a1', 'sub1'])
+        tree.commit('initial commit')
+        self.assertRaises(TypeError, tree.move, ['a1'],
+                          to_this_parameter_does_not_exist='sub1',
+                          after=False)
+
+    def test_move_deprecated_deprecated_call(self):
+        """tree.move has the deprecated parameter 'to_name'.
+        It has been replaced by 'to_dir' for consistency.
+        Test the new API using deprecated parameter"""
+        self.build_tree(['a1', 'sub1/'])
+        tree = self.make_branch_and_tree('.')
+        tree.add(['a1', 'sub1'])
+        tree.commit('initial commit')
+
+        #tree.move(['a1'], to_name='sub1', after=False)
+        self.callDeprecated(['The parameter to_name was deprecated'
+                             ' in version 0.13. Use to_dir instead'],
+                            tree.move, ['a1'], to_name='sub1',
+                            after=False)

=== modified file 'bzrlib/tree.py'
--- bzrlib/tree.py	2006-12-17 17:42:43 +0000
+++ bzrlib/tree.py	2007-01-11 15:57:49 +0000
@@ -141,7 +141,7 @@
         """
         return self.bzrdir.is_control_filename(filename)
 
-    def iter_entries_by_dir(self):
+    def iter_entries_by_dir(self, specific_file_ids=None):
         """Walk the tree in 'by_dir' order.
 
         This will yield each entry in the tree as a (path, entry) tuple. The
@@ -149,7 +149,8 @@
         preceeded by the parent of a directory, and all the contents of a 
         directory are grouped together.
         """
-        return self.inventory.iter_entries_by_dir()
+        return self.inventory.iter_entries_by_dir(
+            specific_file_ids=specific_file_ids)
 
     def kind(self, file_id):
         raise NotImplementedError("subclasses must implement kind")
@@ -476,21 +477,16 @@
         Iteration is done in parent-to-child order, relative to the to_tree.
         """
         to_paths = {}
-        from_entries_by_dir = list(from_tree.inventory.iter_entries_by_dir())
+        from_entries_by_dir = list(from_tree.inventory.iter_entries_by_dir(
+            specific_file_ids=specific_file_ids))
         from_data = dict((e.file_id, (p, e)) for p, e in from_entries_by_dir)
-        to_entries_by_dir = list(to_tree.inventory.iter_entries_by_dir())
-        if specific_file_ids is not None:
-            specific_file_ids = set(specific_file_ids)
-            num_entries = len(specific_file_ids)
-        else:
-            num_entries = len(from_entries_by_dir) + len(to_entries_by_dir)
+        to_entries_by_dir = list(to_tree.inventory.iter_entries_by_dir(
+            specific_file_ids=specific_file_ids))
+        num_entries = len(from_entries_by_dir) + len(to_entries_by_dir)
         entry_count = 0
         for to_path, to_entry in to_entries_by_dir:
             file_id = to_entry.file_id
             to_paths[file_id] = to_path
-            if (specific_file_ids is not None and 
-                file_id not in specific_file_ids):
-                continue
             entry_count += 1
             changed_content = False
             from_path, from_entry = from_data.get(file_id, (None, None))
@@ -501,8 +497,7 @@
                 from_parent = from_entry.parent_id
                 from_kind, from_executable, from_stat = \
                     from_tree._comparison_data(from_entry, from_path)
-                if specific_file_ids is None:
-                    entry_count += 1
+                entry_count += 1
             else:
                 from_versioned = False
                 from_kind = None
@@ -538,19 +533,22 @@
                 yield (file_id, to_path, changed_content, versioned, parent,
                        name, kind, executable)
 
-        for path, from_entry in from_entries_by_dir:
-            file_id = from_entry.file_id
-            if file_id in to_paths:
-                continue
+        def get_to_path(from_entry):
             if from_entry.parent_id is None:
                 to_path = ''
             else:
+                if from_entry.parent_id not in to_paths:
+                    get_to_path(from_tree.inventory[from_entry.parent_id])
                 to_path = osutils.pathjoin(to_paths[from_entry.parent_id],
                                            from_entry.name)
-            to_paths[file_id] = to_path
-            if (specific_file_ids is not None and 
-                file_id not in specific_file_ids):
+            to_paths[from_entry.file_id] = to_path
+            return to_path
+
+        for path, from_entry in from_entries_by_dir:
+            file_id = from_entry.file_id
+            if file_id in to_paths:
                 continue
+            to_path = get_to_path(from_entry)
             entry_count += 1
             if pb is not None:
                 pb.update('comparing files', entry_count, num_entries)

=== modified file 'bzrlib/versionedfile.py'
--- bzrlib/versionedfile.py	2006-10-16 01:25:46 +0000
+++ bzrlib/versionedfile.py	2007-01-17 15:37:08 +0000
@@ -27,6 +27,7 @@
 from bzrlib import (
     errors,
     tsort,
+    revision,
     ui,
     )
 from bzrlib.transport.memory import MemoryTransport
@@ -58,6 +59,10 @@
         self.finished = False
         self._access_mode = access_mode
 
+    @staticmethod
+    def check_not_reserved_id(version_id):
+        revision.check_not_reserved_id(version_id)
+
     def copy_to(self, name, transport):
         """Copy this versioned file to name on transport."""
         raise NotImplementedError(self.copy_to)

=== modified file 'bzrlib/weave.py'
--- bzrlib/weave.py	2006-09-25 14:52:38 +0000
+++ bzrlib/weave.py	2007-01-17 15:37:08 +0000
@@ -247,6 +247,7 @@
 
     def _lookup(self, name):
         """Convert symbolic version name to index."""
+        self.check_not_reserved_id(name)
         try:
             return self._name_map[name]
         except KeyError:
@@ -1120,6 +1121,7 @@
 
     def _add_lines(self, version_id, parents, lines, parent_texts):
         """Add a version and save the weave."""
+        self.check_not_reserved_id(version_id)
         result = super(WeaveFile, self)._add_lines(version_id, parents, lines,
                                                    parent_texts)
         self._save()

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2006-12-19 17:06:42 +0000
+++ bzrlib/workingtree.py	2007-01-20 01:44:46 +0000
@@ -72,16 +72,6 @@
 
 from bzrlib import symbol_versioning
 from bzrlib.decorators import needs_read_lock, needs_write_lock
-from bzrlib.errors import (BzrCheckError,
-                           BzrError,
-                           ConflictFormatError,
-                           WeaveRevisionNotPresent,
-                           NotBranchError,
-                           NoSuchFile,
-                           NotVersionedError,
-                           MergeModifiedFormatError,
-                           UnsupportedOperation,
-                           )
 from bzrlib.inventory import InventoryEntry, Inventory, ROOT_ID
 from bzrlib.lockable_files import LockableFiles, TransportLock
 from bzrlib.lockdir import LockDir
@@ -91,12 +81,12 @@
     compact_date,
     file_kind,
     isdir,
+    normpath,
     pathjoin,
+    rand_chars,
+    realpath,
     safe_unicode,
     splitpath,
-    rand_chars,
-    normpath,
-    realpath,
     supports_executable,
     )
 from bzrlib.trace import mutter, note
@@ -239,8 +229,8 @@
         mutter("opening working tree %r", basedir)
         if deprecated_passed(branch):
             if not _internal:
-                warnings.warn("WorkingTree(..., branch=XXX) is deprecated as of bzr 0.8."
-                     " Please use bzrdir.open_workingtree() or"
+                warnings.warn("WorkingTree(..., branch=XXX) is deprecated"
+                     " as of bzr 0.8. Please use bzrdir.open_workingtree() or"
                      " WorkingTree.open().",
                      DeprecationWarning,
                      stacklevel=2
@@ -265,7 +255,8 @@
         # if needed, or, when the cache sees a change, append it to the hash
         # cache file, and have the parser take the most recent entry for a
         # given path only.
-        cache_filename = self.bzrdir.get_workingtree_transport(None).local_abspath('stat-cache')
+        wt_trans = self.bzrdir.get_workingtree_transport(None)
+        cache_filename = wt_trans.local_abspath('stat-cache')
         self._hashcache = hashcache.HashCache(basedir, cache_filename,
                                               self._control_files._file_mode)
         hc = self._hashcache
@@ -395,7 +386,7 @@
                 if inv is not None and inv.revision_id == revision_id:
                     return bzrlib.revisiontree.RevisionTree(
                         self.branch.repository, inv, revision_id)
-            except (NoSuchFile, errors.BadInventoryFormat):
+            except (errors.NoSuchFile, errors.BadInventoryFormat):
                 pass
         # No cached copy available, retrieve from the repository.
         # FIXME? RBC 20060403 should we cache the inventory locally
@@ -513,7 +504,7 @@
             parents = [last_rev]
         try:
             merges_file = self._control_files.get_utf8('pending-merges')
-        except NoSuchFile:
+        except errors.NoSuchFile:
             pass
         else:
             for l in merges_file.readlines():
@@ -632,7 +623,7 @@
                     kinds[pos] = file_kind(fullpath)
                 except OSError, e:
                     if e.errno == errno.ENOENT:
-                        raise NoSuchFile(fullpath)
+                        raise errors.NoSuchFile(fullpath)
 
     @needs_write_lock
     def add_parent_tree_id(self, revision_id, allow_leftmost_as_ghost=False):
@@ -647,8 +638,8 @@
         :param allow_leftmost_as_ghost: Allow the first parent to be a ghost.
         """
         parents = self.get_parent_ids() + [revision_id]
-        self.set_parent_ids(parents,
-            allow_leftmost_as_ghost=len(parents) > 1 or allow_leftmost_as_ghost)
+        self.set_parent_ids(parents, allow_leftmost_as_ghost=len(parents) > 1
+            or allow_leftmost_as_ghost)
 
     @needs_tree_write_lock
     def add_parent_tree(self, parent_tuple, allow_leftmost_as_ghost=False):
@@ -788,9 +779,9 @@
         """Merge from a branch into this working tree.
 
         :param branch: The branch to merge from.
-        :param to_revision: If non-None, the merge will merge to to_revision, but 
-            not beyond it. to_revision does not need to be in the history of
-            the branch when it is supplied. If None, to_revision defaults to
+        :param to_revision: If non-None, the merge will merge to to_revision,
+            but not beyond it. to_revision does not need to be in the history
+            of the branch when it is supplied. If None, to_revision defaults to
             branch.last_revision().
         """
         from bzrlib.merge import Merger, Merge3Merger
@@ -830,14 +821,14 @@
     def merge_modified(self):
         try:
             hashfile = self._control_files.get('merge-hashes')
-        except NoSuchFile:
+        except errors.NoSuchFile:
             return {}
         merge_hashes = {}
         try:
             if hashfile.next() != MERGE_MODIFIED_HEADER_1 + '\n':
-                raise MergeModifiedFormatError()
+                raise errors.MergeModifiedFormatError()
         except StopIteration:
-            raise MergeModifiedFormatError()
+            raise errors.MergeModifiedFormatError()
         for s in RioReader(hashfile):
             file_id = s.get("file_id")
             if file_id not in self.inventory:
@@ -961,9 +952,9 @@
 
                 if f_ie:
                     if f_ie.kind != fk:
-                        raise BzrCheckError("file %r entered as kind %r id %r, "
-                                            "now of kind %r"
-                                            % (fap, f_ie.kind, f_ie.file_id, fk))
+                        raise errors.BzrCheckError(
+                            "file %r entered as kind %r id %r, now of kind %r"
+                            % (fap, f_ie.kind, f_ie.file_id, fk))
 
                 # make a last minute entry
                 if f_ie:
@@ -983,130 +974,292 @@
                 new_children.sort()
                 new_children = collections.deque(new_children)
                 stack.append((f_ie.file_id, fp, fap, new_children))
-                # Break out of inner loop, so that we start outer loop with child
+                # Break out of inner loop,
+                # so that we start outer loop with child
                 break
             else:
                 # if we finished all children, pop it off the stack
                 stack.pop()
 
     @needs_tree_write_lock
-    def move(self, from_paths, to_name):
+    def move(self, from_paths, to_dir=None, after=False, **kwargs):
         """Rename files.
 
-        to_name must exist in the inventory.
+        to_dir must exist in the inventory.
 
-        If to_name exists and is a directory, the files are moved into
+        If to_dir exists and is a directory, the files are moved into
         it, keeping their old names.  
 
-        Note that to_name is only the last component of the new name;
+        Note that to_dir is only the last component of the new name;
         this doesn't change the directory.
 
+        For each entry in from_paths the move mode will be determined
+        independently.
+
+        The first mode moves the file in the filesystem and updates the
+        inventory. The second mode only updates the inventory without
+        touching the file on the filesystem. This is the new mode introduced
+        in version 0.15.
+
+        move uses the second mode if 'after == True' and the target is not
+        versioned but present in the working tree.
+
+        move uses the second mode if 'after == False' and the source is
+        versioned but no longer in the working tree, and the target is not
+        versioned but present in the working tree.
+
+        move uses the first mode if 'after == False' and the source is
+        versioned and present in the working tree, and the target is not
+        versioned and not present in the working tree.
+
+        Everything else results in an error.
+
         This returns a list of (from_path, to_path) pairs for each
         entry that is moved.
         """
-        result = []
-        ## TODO: Option to move IDs only
+        rename_entries = []
+        rename_tuples = []
+
+        # check for deprecated use of signature
+        if to_dir is None:
+            to_dir = kwargs.get('to_name', None)
+            if to_dir is None:
+                raise TypeError('You must supply a target directory')
+            else:
+                symbol_versioning.warn('The parameter to_name was deprecated'
+                                       ' in version 0.13. Use to_dir instead',
+                                       DeprecationWarning)
+
+        # check destination directory
         assert not isinstance(from_paths, basestring)
         inv = self.inventory
-        to_abs = self.abspath(to_name)
+        to_abs = self.abspath(to_dir)
         if not isdir(to_abs):
-            raise BzrError("destination %r is not a directory" % to_abs)
-        if not self.has_filename(to_name):
-            raise BzrError("destination %r not in working directory" % to_abs)
-        to_dir_id = inv.path2id(to_name)
-        if to_dir_id is None and to_name != '':
-            raise BzrError("destination %r is not a versioned directory" % to_name)
+            raise errors.BzrMoveFailedError('',to_dir,
+                errors.NotADirectory(to_abs))
+        if not self.has_filename(to_dir):
+            raise errors.BzrMoveFailedError('',to_dir,
+                errors.NotInWorkingDirectory(to_dir))
+        to_dir_id = inv.path2id(to_dir)
+        if to_dir_id is None:
+            raise errors.BzrMoveFailedError('',to_dir,
+                errors.NotVersionedError(path=str(to_dir)))
+
         to_dir_ie = inv[to_dir_id]
         if to_dir_ie.kind != 'directory':
-            raise BzrError("destination %r is not a directory" % to_abs)
-
-        to_idpath = inv.get_idpath(to_dir_id)
-
-        for f in from_paths:
-            if not self.has_filename(f):
-                raise BzrError("%r does not exist in working tree" % f)
-            f_id = inv.path2id(f)
-            if f_id is None:
-                raise BzrError("%r is not versioned" % f)
-            name_tail = splitpath(f)[-1]
-            dest_path = pathjoin(to_name, name_tail)
-            if self.has_filename(dest_path):
-                raise BzrError("destination %r already exists" % dest_path)
-            if f_id in to_idpath:
-                raise BzrError("can't move %r to a subdirectory of itself" % f)
-
-        # OK, so there's a race here, it's possible that someone will
-        # create a file in this interval and then the rename might be
-        # left half-done.  But we should have caught most problems.
-        orig_inv = deepcopy(self.inventory)
+            raise errors.BzrMoveFailedError('',to_dir,
+                errors.NotADirectory(to_abs))
+
+        # create rename entries and tuples
+        for from_rel in from_paths:
+            from_tail = splitpath(from_rel)[-1]
+            from_id = inv.path2id(from_rel)
+            if from_id is None:
+                raise errors.BzrMoveFailedError(from_rel,to_dir,
+                    errors.NotVersionedError(path=str(from_rel)))
+
+            from_entry = inv[from_id]
+            from_parent_id = from_entry.parent_id
+            to_rel = pathjoin(to_dir, from_tail)
+            rename_entry = WorkingTree._RenameEntry(from_rel=from_rel,
+                                         from_id=from_id,
+                                         from_tail=from_tail,
+                                         from_parent_id=from_parent_id,
+                                         to_rel=to_rel, to_tail=from_tail,
+                                         to_parent_id=to_dir_id)
+            rename_entries.append(rename_entry)
+            rename_tuples.append((from_rel, to_rel))
+
+        # determine which move mode to use. checks also for movability
+        rename_entries = self._determine_mv_mode(rename_entries, after)
+
         original_modified = self._inventory_is_modified
         try:
             if len(from_paths):
                 self._inventory_is_modified = True
-            for f in from_paths:
-                name_tail = splitpath(f)[-1]
-                dest_path = pathjoin(to_name, name_tail)
-                result.append((f, dest_path))
-                inv.rename(inv.path2id(f), to_dir_id, name_tail)
-                try:
-                    osutils.rename(self.abspath(f), self.abspath(dest_path))
-                except OSError, e:
-                    raise BzrError("failed to rename %r to %r: %s" %
-                                   (f, dest_path, e[1]))
+            self._move(rename_entries)
         except:
             # restore the inventory on error
-            self._set_inventory(orig_inv, dirty=original_modified)
+            self._inventory_is_modified = original_modified
             raise
         self._write_inventory(inv)
-        return result
+        return rename_tuples
+
+    def _determine_mv_mode(self, rename_entries, after=False):
+        """Determines for each from-to pair if both inventory and working tree
+        or only the inventory has to be changed.
+
+        Also does basic plausability tests.
+        """
+        inv = self.inventory
+
+        for rename_entry in rename_entries:
+            # store to local variables for easier reference
+            from_rel = rename_entry.from_rel
+            from_id = rename_entry.from_id
+            to_rel = rename_entry.to_rel
+            to_id = inv.path2id(to_rel)
+            only_change_inv = False
+
+            # check the inventory for source and destination
+            if from_id is None:
+                raise errors.BzrMoveFailedError(from_rel,to_rel,
+                    errors.NotVersionedError(path=str(from_rel)))
+            if to_id is not None:
+                raise errors.BzrMoveFailedError(from_rel,to_rel,
+                    errors.AlreadyVersionedError(path=str(to_rel)))
+
+            # try to determine the mode for rename (only change inv or change
+            # inv and file system)
+            if after:
+                if not self.has_filename(to_rel):
+                    raise errors.BzrMoveFailedError(from_id,to_rel,
+                        errors.NoSuchFile(path=str(to_rel),
+                        extra="New file has not been created yet"))
+                only_change_inv = True
+            elif not self.has_filename(from_rel) and self.has_filename(to_rel):
+                only_change_inv = True
+            elif self.has_filename(from_rel) and not self.has_filename(to_rel):
+                only_change_inv = False
+            else:
+                # something is wrong, so lets determine what exactly
+                if not self.has_filename(from_rel) and \
+                   not self.has_filename(to_rel):
+                    raise errors.BzrRenameFailedError(from_rel,to_rel,
+                        errors.PathsDoNotExist(paths=(str(from_rel),
+                        str(to_rel))))
+                else:
+                    raise errors.RenameFailedFilesExist(from_rel, to_rel,
+                        extra="(Use --after to update the Bazaar id)")
+            rename_entry.only_change_inv = only_change_inv
+        return rename_entries
+
+    def _move(self, rename_entries):
+        """Moves a list of files.
+
+        Depending on the value of the flag 'only_change_inv', the
+        file will be moved on the file system or not.
+        """
+        inv = self.inventory
+        moved = []
+
+        for entry in rename_entries:
+            try:
+                self._move_entry(entry)
+            except:
+                self._rollback_move(moved)
+                raise
+            moved.append(entry)
+
+    def _rollback_move(self, moved):
+        """Try to rollback a previous move in case of an filesystem error."""
+        inv = self.inventory
+        for entry in moved:
+            try:
+                self._move_entry(_RenameEntry(entry.to_rel, entry.from_id,
+                    entry.to_tail, entry.to_parent_id, entry.from_rel,
+                    entry.from_tail, entry.from_parent_id,
+                    entry.only_change_inv))
+            except errors.BzrMoveFailedError, e:
+                raise errors.BzrMoveFailedError( '', '', "Rollback failed."
+                        " The working tree is in an inconsistent state."
+                        " Please consider doing a 'bzr revert'."
+                        " Error message is: %s" % e)
+
+    def _move_entry(self, entry):
+        inv = self.inventory
+        from_rel_abs = self.abspath(entry.from_rel)
+        to_rel_abs = self.abspath(entry.to_rel)
+        if from_rel_abs == to_rel_abs:
+            raise errors.BzrMoveFailedError(entry.from_rel, entry.to_rel,
+                "Source and target are identical.")
+
+        if not entry.only_change_inv:
+            try:
+                osutils.rename(from_rel_abs, to_rel_abs)
+            except OSError, e:
+                raise errors.BzrMoveFailedError(entry.from_rel,
+                    entry.to_rel, e[1])
+        inv.rename(entry.from_id, entry.to_parent_id, entry.to_tail)
 
     @needs_tree_write_lock
-    def rename_one(self, from_rel, to_rel):
+    def rename_one(self, from_rel, to_rel, after=False):
         """Rename one file.
 
         This can change the directory or the filename or both.
+
+        rename_one has several 'modes' to work. First, it can rename a physical
+        file and change the file_id. That is the normal mode. Second, it can
+        only change the file_id without touching any physical file. This is
+        the new mode introduced in version 0.15.
+
+        rename_one uses the second mode if 'after == True' and 'to_rel' is not
+        versioned but present in the working tree.
+
+        rename_one uses the second mode if 'after == False' and 'from_rel' is
+        versioned but no longer in the working tree, and 'to_rel' is not
+        versioned but present in the working tree.
+
+        rename_one uses the first mode if 'after == False' and 'from_rel' is
+        versioned and present in the working tree, and 'to_rel' is not
+        versioned and not present in the working tree.
+
+        Everything else results in an error.
         """
         inv = self.inventory
-        if not self.has_filename(from_rel):
-            raise BzrError("can't rename: old working file %r does not exist" % from_rel)
-        if self.has_filename(to_rel):
-            raise BzrError("can't rename: new working file %r already exists" % to_rel)
-
-        file_id = inv.path2id(from_rel)
-        if file_id is None:
-            raise BzrError("can't rename: old name %r is not versioned" % from_rel)
-
-        entry = inv[file_id]
-        from_parent = entry.parent_id
-        from_name = entry.name
-        
-        if inv.path2id(to_rel):
-            raise BzrError("can't rename: new name %r is already versioned" % to_rel)
-
+        rename_entries = []
+
+        # create rename entries and tuples
+        from_tail = splitpath(from_rel)[-1]
+        from_id = inv.path2id(from_rel)
+        if from_id is None:
+            raise errors.BzrRenameFailedError(from_rel,to_rel,
+                errors.NotVersionedError(path=str(from_rel)))
+        from_entry = inv[from_id]
+        from_parent_id = from_entry.parent_id
         to_dir, to_tail = os.path.split(to_rel)
         to_dir_id = inv.path2id(to_dir)
-        if to_dir_id is None and to_dir != '':
-            raise BzrError("can't determine destination directory id for %r" % to_dir)
-
-        mutter("rename_one:")
-        mutter("  file_id    {%s}" % file_id)
-        mutter("  from_rel   %r" % from_rel)
-        mutter("  to_rel     %r" % to_rel)
-        mutter("  to_dir     %r" % to_dir)
-        mutter("  to_dir_id  {%s}" % to_dir_id)
-
-        inv.rename(file_id, to_dir_id, to_tail)
-
-        from_abs = self.abspath(from_rel)
-        to_abs = self.abspath(to_rel)
-        try:
-            osutils.rename(from_abs, to_abs)
-        except OSError, e:
-            inv.rename(file_id, from_parent, from_name)
-            raise BzrError("failed to rename %r to %r: %s"
-                    % (from_abs, to_abs, e[1]))
+        rename_entry = WorkingTree._RenameEntry(from_rel=from_rel,
+                                     from_id=from_id,
+                                     from_tail=from_tail,
+                                     from_parent_id=from_parent_id,
+                                     to_rel=to_rel, to_tail=to_tail,
+                                     to_parent_id=to_dir_id)
+        rename_entries.append(rename_entry)
+
+        # determine which move mode to use. checks also for movability
+        rename_entries = self._determine_mv_mode(rename_entries, after)
+
+        # check if the target changed directory and if the target directory is
+        # versioned
+        if to_dir_id is None:
+            raise errors.BzrMoveFailedError(from_rel,to_rel,
+                errors.NotVersionedError(path=str(to_dir)))
+
+        # all checks done. now we can continue with our actual work
+        mutter('rename_one:\n'
+               '  from_id   {%s}\n'
+               '  from_rel: %r\n'
+               '  to_rel:   %r\n'
+               '  to_dir    %r\n'
+               '  to_dir_id {%s}\n',
+               from_id, from_rel, to_rel, to_dir, to_dir_id)
+
+        self._move(rename_entries)
         self._write_inventory(inv)
 
+    class _RenameEntry(object):
+        def __init__(self, from_rel, from_id, from_tail, from_parent_id,
+                     to_rel, to_tail, to_parent_id, only_change_inv=False):
+            self.from_rel = from_rel
+            self.from_id = from_id
+            self.from_tail = from_tail
+            self.from_parent_id = from_parent_id
+            self.to_rel = to_rel
+            self.to_tail = to_tail
+            self.to_parent_id = to_parent_id
+            self.only_change_inv = only_change_inv
+
     @needs_read_lock
     def unknowns(self):
         """Return all unknown files.
@@ -1490,7 +1643,7 @@
             if not fid:
                 # TODO: Perhaps make this just a warning, and continue?
                 # This tends to happen when 
-                raise NotVersionedError(path=f)
+                raise errors.NotVersionedError(path=f)
             if verbose:
                 # having remove it, it must be either ignored or unknown
                 if self.is_ignored(f):
@@ -1539,7 +1692,7 @@
             elif kind == 'symlink':
                 inv.add(InventoryLink(file_id, name, parent))
             else:
-                raise BzrError("unknown kind %r" % kind)
+                raise errors.BzrError("unknown kind %r" % kind)
         self._write_inventory(inv)
 
     @needs_tree_write_lock
@@ -1720,10 +1873,10 @@
         self.flush()
 
     def set_conflicts(self, arg):
-        raise UnsupportedOperation(self.set_conflicts, self)
+        raise errors.UnsupportedOperation(self.set_conflicts, self)
 
     def add_conflicts(self, arg):
-        raise UnsupportedOperation(self.add_conflicts, self)
+        raise errors.UnsupportedOperation(self.add_conflicts, self)
 
     @needs_read_lock
     def conflicts(self):
@@ -1803,7 +1956,7 @@
         """See Mutable.last_revision."""
         try:
             return self._control_files.get_utf8('last-revision').read()
-        except NoSuchFile:
+        except errors.NoSuchFile:
             return None
 
     def _change_last_revision(self, revision_id):
@@ -1834,13 +1987,13 @@
     def conflicts(self):
         try:
             confile = self._control_files.get('conflicts')
-        except NoSuchFile:
+        except errors.NoSuchFile:
             return _mod_conflicts.ConflictList()
         try:
             if confile.next() != CONFLICT_HEADER_1 + '\n':
-                raise ConflictFormatError()
+                raise errors.ConflictFormatError()
         except StopIteration:
-            raise ConflictFormatError()
+            raise errors.ConflictFormatError()
         return _mod_conflicts.ConflictList.from_stanzas(RioReader(confile))
 
     def unlock(self):
@@ -1908,7 +2061,7 @@
             transport = a_bzrdir.get_workingtree_transport(None)
             format_string = transport.get("format").read()
             return klass._formats[format_string]
-        except NoSuchFile:
+        except errors.NoSuchFile:
             raise errors.NoWorkingTree(base=transport.base)
         except KeyError:
             raise errors.UnknownFormatError(format=format_string)

=== modified file 'setup.py'
--- setup.py	2006-09-08 18:46:29 +0000
+++ setup.py	2007-01-15 03:31:34 +0000
@@ -188,12 +188,18 @@
                                      version = version_str,
                                      description = META_INFO['description'],
                                      author = META_INFO['author'],
-                                     copyright = "(c) Canonical Ltd, 2005-2006",
+                                     copyright = "(c) Canonical Ltd, 2005-2007",
                                      company_name = "Canonical Ltd.",
                                      comments = META_INFO['description'],
                                     )
+
+    additional_packages =  []
+    if sys.version.startswith('2.4'):
+        # adding elementtree package
+        additional_packages.append('elementtree')
+
     options_list = {"py2exe": {"packages": BZRLIB['packages'] +
-                                           ['elementtree'],
+                                           additional_packages,
                                "excludes": ["Tkinter", "medusa"],
                                "dist_dir": "win32_bzr.exe",
                               },

=== modified file 'tools/win32/bzr.iss.cog'
--- tools/win32/bzr.iss.cog	2006-10-14 08:51:07 +0000
+++ tools/win32/bzr.iss.cog	2007-01-13 20:00:38 +0000
@@ -7,6 +7,14 @@
 AppName=Bazaar
 
 ; [[[cog
+; # Python 2.5 compatibility code
+; import os
+; import sys
+; cwd = os.getcwd()
+; if cwd not in sys.path:
+;     sys.path.insert(0, cwd)
+; #/Python 2.5 compatibility code
+;
 ; import bzrlib
 ; try: 
 ;     VERSION = bzrlib.__version__

